/*  
** Program to test linearity against LSTAR and ESTAR alternatives.
** Data is seasonally UNadjusted unemployment rate males 20+
** Sample period: January 1954 - April 1998
**
** Standard and robust nonlinearity tests are applied
**
** Author: Dick van Dijk
** Last revision on 22 March 2000
*/
new; cls;
library pgraph;
trap 1;

#include robust.prc;
#include strtests.prc;

p_max=18;                               /* Maximum autoregressive order */
p_lag=15;                               /* Actual autoregressive order */
u_dif=1;                                /* u_dif=1 if model is for first differences, 
                                           u_dif=0 if model is for levels */
i_lev=1;                                /* i_lev=1 if lagged level is to be included, 
                                           i_lev=0 if not (NB only relevant if u_dif=1)*/

l_dif=12;                               /* value of "long difference" which is used as transition variable */
max_yd=6;                               /* maximum delay of transition variable */
yd_ind=seqa(1,1,max_yd);   
n_yd=rows(yd_ind);

/*
** Specify sample period of data in files
*/
n_fy=48;                                /* Year of first observation */
n_fm=1;                                 /* Month of first observation */
n_ly=99;                                /* Year of last observation */
n_lm=12;                                /* Month of last observation */
d_nobs=12*(n_ly-n_fy)+n_lm-n_fm+1;      /* Number of observations */

/*
** Specify sample period for which model is to be estimated
*/
e_fy=68; /* year of first observation */
e_fm=6;  /* month of first observation */
e_ly=89; /* year of last observation */
e_lm=12; /* month of last observation */

/* Create monthly dummies */
m_dum=ones(n_ly-n_fy+1,1).*.eye(12);
m_dum=m_dum[n_fm:n_fm+d_nobs-1,.];

/* load raw data */
load lhmu[]=lhmunsa.dat; /* number of unemployed */
load lhmc[]=lhmcnsa.dat; /* labor force */
/* compute unemployment rate */
y_raw=(lhmu./lhmc)*100;

e_nobs=12*(e_ly-e_fy)+e_lm-e_fm+1; /* Number of observations in estimation period */

tdat=packr(y_raw[12*(e_fy-n_fy)+e_fm-n_fm+1:12*(e_fy-n_fy)+e_fm-n_fm+e_nobs]~
           m_dum[12*(e_fy-n_fy)+e_fm-n_fm+1:12*(e_fy-n_fy)+e_fm-n_fm+e_nobs,.]);
y_lev=tdat[.,1];
m_dum=tdat[.,2:13];
T=rows(y_lev);

if u_dif; /* model for first differences */
  y_dif=(y_lev[2:T]-y_lev[1:T-1]); 
  m_dum=m_dum[2:T,.];
  T=T-1;  
  if i_lev; /* include lagged level term */
    deter=ones(T,1)~m_dum~y_lev[1:T];
  else;
    deter=ones(T,1)~m_dum;
    endif;
else; /* model for levels */
  y_dif=y_lev;
  deter=ones(T,1)~m_dum~seqa(1,1,T);
endif;

/* Construct data matrix */
_dat=trimr(lagn(y_dif,seqa(0,1,p_max+1)),p_max,0);
j=0;
do until (j==max_yd);
  j=j+1;
  if u_dif;
    _dat=_dat~sumc(_dat[.,1+j:1+j+l_dif-1]');
  else;
    _dat=_dat~(_dat[.,1+j]-_dat[.,1+j+l_dif]);
  endif;
endo;
_dat=ones(T-p_max,1)~m_dum[p_max+1:T,.]~seqa(1/(T-p_max),1/(T-p_max),T-p_max)~_dat;
if i_lev;
  _dat=_dat~y_lev[p_max+1:T];
endif;
T=rows(_dat);

/* get names for variables in _dat matrix */
names=GETNAMES(u_dif,i_lev,p_max,max_yd);

yi=15;
/*
** Both dummies and lags are tested for nonlinearity 
*/
if p_lag GT 0;
  xli=seqa(1,1,13-u_dif)|seqa(16,1,p_lag);
  xnli=seqa(1,1,13-u_dif)|seqa(16,1,p_lag);
else;
  xli=seqa(1,1,13-u_dif);
  xnli=seqa(1,1,13-u_dif);
endif;
if i_lev;
  xli=xli|(14+1+p_max+max_yd+1);
  xnli=xnli|(14+1+p_max+max_yd+1);
endif;
cli=1;
cnli=1;
detli=seqa(1,1,12);
detnli=seqa(1,1,12);


/* indices of transition variable - 14=time trend */
tr_ind=14|seqa(14+1+p_max+1,1,max_yd);
ntr=rows(tr_ind);
res=0;

LM_1=zeros(2*ntr,6);
LM_2=zeros(2*ntr,6);
LM_3=zeros(2*ntr,6);
LM_4=zeros(2*ntr,6);
LM_3e=zeros(2*ntr,6);

LM_H4=zeros(2*ntr,6);
LM_H3=zeros(2*ntr,6);
LM_H2=zeros(2*ntr,6);
LM_H1=zeros(2*ntr,6);
LM_HL=zeros(2*ntr,6);
LM_HE=zeros(2*ntr,6);

tim=date;
d=0;
do until (d==ntr);
  d=d+1;
  xtri = tr_ind[d];
  "";"Transition variable " $names[xtri] ;"";
  if (d .LE p_lag-l_dif+u_dif+1);
    exc=1;
  else; 
    exc=0; 
  endif;

  /* Standard test statistics */
  {LM_1[2*d-1:2*d,1],LM_1[2*d-1:2*d,2],LM_2[2*d-1:2*d,1],LM_2[2*d-1:2*d,2],
   LM_3[2*d-1:2*d,1],LM_3[2*d-1:2*d,2],LM_3e[2*d-1:2*d,1],LM_3e[2*d-1:2*d,2],
   LM_4[2*d-1:2*d,1],LM_4[2*d-1:2*d,2]}=
      SSTRTX(_dat,yi,xli,cli,xnli,cnli,xtri,exc,0);

  {t1,t2,t3,t4,
   LM_H4[2*d-1:2*d,1],LM_H4[2*d-1:2*d,2],LM_H3[2*d-1:2*d,1],LM_H3[2*d-1:2*d,2],
   LM_H2[2*d-1:2*d,1],LM_H2[2*d-1:2*d,2],LM_H1[2*d-1:2*d,1],LM_H1[2*d-1:2*d,2],
   LM_HL[2*d-1:2*d,1],LM_HL[2*d-1:2*d,2],LM_HE[2*d-1:2*d,1],LM_HE[2*d-1:2*d,2]}=
      SSTRSPA(_dat,yi,xli,cli,xnli,cnli,xtri,exc,0);

  /* Heteroskedasticity-robust test statistics */
  {LM_1[2*d-1:2*d,3],LM_1[2*d-1:2*d,4],LM_2[2*d-1:2*d,3],LM_2[2*d-1:2*d,4],
   LM_3[2*d-1:2*d,3],LM_3[2*d-1:2*d,4],LM_3e[2*d-1:2*d,3],LM_3e[2*d-1:2*d,4],
   LM_4[2*d-1:2*d,3],LM_4[2*d-1:2*d,4]}=
      WSTRTX(_dat,yi,xli,cli,xnli,cnli,xtri,exc,0);

  {t1,t2,t3,t4,
   LM_H4[2*d-1:2*d,3],LM_H4[2*d-1:2*d,4],LM_H3[2*d-1:2*d,3],LM_H3[2*d-1:2*d,4],
   LM_H2[2*d-1:2*d,3],LM_H2[2*d-1:2*d,4],LM_H1[2*d-1:2*d,3],LM_H1[2*d-1:2*d,4],
   LM_HL[2*d-1:2*d,3],LM_HL[2*d-1:2*d,4],LM_HE[2*d-1:2*d,3],LM_HE[2*d-1:2*d,4]}=
      WSTRSPA(_dat,yi,xli,cli,xnli,cnli,xtri,exc,0);

  /* Outlier-robust test statistics */
  {LM_1[2*d-1:2*d,5],LM_1[2*d-1:2*d,6],LM_2[2*d-1:2*d,5],LM_2[2*d-1:2*d,6],
   LM_3[2*d-1:2*d,5],LM_3[2*d-1:2*d,6],LM_3e[2*d-1:2*d,5],LM_3e[2*d-1:2*d,6],
   LM_4[2*d-1:2*d,5],LM_4[2*d-1:2*d,6],weights0}=
      RSTRTX(_dat,yi,xli,cli,detli,xnli,cnli,detnli,xtri,exc,0,0);

  {t1,t2,t3,t4,
   LM_H4[2*d-1:2*d,5],LM_H4[2*d-1:2*d,6],LM_H3[2*d-1:2*d,5],LM_H3[2*d-1:2*d,6],
   LM_H2[2*d-1:2*d,5],LM_H2[2*d-1:2*d,6],LM_H1[2*d-1:2*d,5],LM_H1[2*d-1:2*d,6],
   LM_HL[2*d-1:2*d,5],LM_HL[2*d-1:2*d,6],LM_HE[2*d-1:2*d,5],LM_HE[2*d-1:2*d,6],weightse}=
    RSTRSPA(_dat,yi,xli,cli,detli,xnli,cnli,detnli,xtri,exc,0,0);

  if d==1; /* save weights from outlier-robust estimation */
    fname = "URNSAWGT.OUT";
    output file = ^fname reset ; screen off;
      weights0;
    output off; screen on;
  endif;

  info(d,ntr,tim);
endo;


ff="#*.*lf"~12~3;
ff1="*.*lG"~12~3;
ff2="#*.*lf"~12~7;
outwidth 120;
fname = "URNSATST.OUT";
output file = ^fname reset ; screen off;
"Number of observations " T;
"AR order p:" p_lag;"";

"";"Chi-square variants";
d=0;
do until (d==ntr);
  d=d+1; ds=2*d-1;
  xtri = tr_ind[d];

  "";"Transition variable " $names[xtri] ;"";
  call printfm("Test"~"LS"~"pval"~"LSW"~"pval"~"GM"~"pval",zeros(1,7),ff);"";
  call printfm("LM1"~LM_1[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";
  call printfm("LM2"~LM_2[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";
  call printfm("LM3"~LM_3[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";
  call printfm("LM3e"~LM_3e[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";
  call printfm("LM4"~LM_4[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";

  "";call printfm("H4"~LM_H4[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";
  call printfm("H3"~LM_H3[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";
  call printfm("H2"~LM_H2[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";
  call printfm("H1"~LM_H1[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";
  
  "";call printfm("HL"~LM_HL[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";
  call printfm("HE"~LM_HE[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";
endo;

"";"F-variants";
d=0;
do until (d==ntr) ;
  d=d+1; ds=2*d;
  xtri = tr_ind[d];

  "";"Transition variable " $names[xtri] ;"";
  call printfm("Test"~"LS"~"pval"~"LSW"~"pval"~"GM"~"pval",zeros(1,7),ff);"";
  call printfm("LM1"~LM_1[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";
  call printfm("LM2"~LM_2[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";
  call printfm("LM3"~LM_3[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";
  call printfm("LM3e"~LM_3e[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";
  call printfm("LM4"~LM_4[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";

  "";call printfm("H4"~LM_H4[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";
  call printfm("H3"~LM_H3[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";
  call printfm("H2"~LM_H2[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";
  call printfm("H1"~LM_H1[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";
  
  "";call printfm("HL"~LM_HL[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";
  call printfm("HE"~LM_HE[ds,.],0~ones(1,6),ff2|ff|ff1|ff|ff1|ff|ff1);"";
endo;
output off ; screen on;

/*
** GETNAMES.g
**
** construct names of variables in _dat matrix
**
*/
proc(1)=GETNAMES(u_dif,i_lev,p_max,max_yd);
  local names,namej,l,pn;

names="const"|"M1"|"M2"|"M3"|"M4"|"M5"|"M6"|"M7"|"M8"|"M9"|"M10"|"M11"|"M12"|"trend";
if (u_dif);
  names=names|"dy(t)";
else;
  names=names|"y(t)";
endif;
l=0;
do until (l==p_max);
  l=l+1;
  if l<10; pn=ftocv(l,1,0); else; pn=ftocv(l,2,0); endif;
  if u_dif;
    namej="" $+ "dy(t-" $+ pn $+ ")";
  else;
    namej="" $+ "y(t-" $+ pn $+ ")";
  endif;
  names=names|namej;
endo;
l=0;
do until (l==max_yd);
  l=l+1;
  if l<10; pn=ftocv(l,1,0); else; pn=ftocv(l,2,0); endif;
  namej="dy*(t-" $+ pn $+ ")";
  names=names|namej;
endo;
if (i_lev);
  names=names|"y(t-1)";
endif;

retp(names);
endp;

